package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
	"github.com/pivotal-cf/on-demand-service-broker-sdk/serviceadapter"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
)

func (b *Broker) Unbind(instanceID, bindingID string, details brokerapi.UnbindDetails) error {
	logger := b.loggerFactory.NewLogger()

	errs := func(context string, err error) error {
		logger.Printf("error %s: %s", context, err)
		return convertToDisplayableError(err, logger)
	}

	vms, manifest, err := b.getDeploymentInfo(instanceID, logger)
	switch err.(type) {
	case boshclient.DeploymentNotFoundError:
		logger.Println(fmt.Errorf("error unbinding: instance %s, not found", instanceID))
		return brokerapi.ErrInstanceDoesNotExist
	case error:
		return errs("gathering unbinding info", err)
	}

	requestParams := map[string]interface{}{
		"plan_id":    details.PlanID,
		"service_id": details.ServiceID,
	}

	logger.Printf("service adapter will delete binding with ID %s for instance %s\n", bindingID, instanceID)
	bindingErr := b.serviceAdapter.DeleteBinding(bindingID, vms, manifest, requestParams, logger)
	switch bindingErr.(type) {
	case serviceadapter.BindingNotFoundError:
		return errs("deleting binding", DisplayableError{brokerapi.ErrBindingDoesNotExist})
	case error:
		return errs("deleting binding", bindingErr)
	}

	return nil
}
